@@ -11,6 +11,7 @@ use File::pfopen 0.02;
 use File::Temp;
 use Gzip::Faster;
 use DBD::SQLite::Constants qw/:file_open/;	# For SQLITE_OPEN_READONLY
+use Error::Simple;
 
 our @databases;
 our $directory;
@@ -321,7 +322,11 @@ sub AUTOLOAD {
 	}
 	$query .= " ORDER BY $column";
 	if($self->{'logger'}) {
-		$self->{'logger'}->debug("AUTOLOAD $query: " . join(', ', @args));
+		if(scalar(@args)) {
+			$self->{'logger'}->debug("AUTOLOAD $query: " . join(', ', @args));
+		} else {
+			$self->{'logger'}->debug("AUTOLOAD $query");
+		}
 	}
 	my $sth = $self->{$table}->prepare($query) || throw Error::Simple($query);
 	$sth->execute(@args) || throw Error::Simple($query);
