@@ -1,5 +1,8 @@
 package Geo::Coder::Free::DB::admin2;
 
+use strict;
+use warnings;
+
 # admin2.db is from http://download.geonames.org/export/dump/admin2Codes.txt
 
 use Geo::Coder::Free::DB;
